<?php

declare(strict_types=1);
/**
 * Shopware 5
 * Copyright (c) shopware AG
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Shopware" is a registered trademark of shopware AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 */

namespace Shopware\Tests\Mink\Tests\Frontend\Account\bootstrap;

use Behat\Gherkin\Node\TableNode;
use Behat\Mink\WebAssert;
use Doctrine\DBAL\Connection;
use Shopware\Tests\Mink\Page\Frontend\Account\Account;
use Shopware\Tests\Mink\Page\Frontend\Address\Address;
use Shopware\Tests\Mink\Page\Frontend\Address\AddressDelete;
use Shopware\Tests\Mink\Page\Frontend\Address\AddressEdit;
use Shopware\Tests\Mink\Page\Frontend\Address\Elements\AddressManagementAddressBox;
use Shopware\Tests\Mink\Page\Frontend\Checkout\CheckoutConfirm;
use Shopware\Tests\Mink\Tests\General\Helpers\Helper;
use Shopware\Tests\Mink\Tests\General\Helpers\SubContext;

class AccountContext extends SubContext
{
    /**
     * @Given /^I log in with email "(?P<email>[^"]*)" and password "(?P<password>[^"]*)"$/
     */
    public function iLogInAsWithPassword($email, $password)
    {
        $this->getPage('Account')->login($email, $password);
    }

    /**
     * @Given /^I log in successful as "(?P<username>[^"]*)" with email "(?P<email>[^"]*)" and password "(?P<password>[^"]*)"$/
     */
    public function iLogInSuccessfulAsWithPassword($username, $email, $password)
    {
        Shopware()->Container()->get(Connection::class)->executeQuery('DELETE FROM s_order_basket');

        $this->getPage('Account')->login($email, $password);
        $this->getPage('Account')->verifyLogin($username);
    }

    /**
     * @When /^I log me out$/
     */
    public function iLogMeOut()
    {
        $this->getPage('Account')->logout();
    }

    /**
     * @When /^I click on login again$/
     */
    public function iClickOnLoginAgain()
    {
        $this->getPage('Account')->clickLoginAgain();
    }

    /**
     * @Then /^I change my email with password "(?P<password>[^"]*)" to "(?P<new>[^"]*)"$/
     * @Then /^I change my email with password "(?P<password>[^"]*)" to "(?P<new>[^"]*)" with confirmation "(?P<confirmation>[^"]*)"$/
     */
    public function iChangeMyEmailWithPasswordToWithConfirmation($password, $email, $emailConfirmation = null)
    {
        $this->getPage('Account')->changeEmail($password, $email, $emailConfirmation);
    }

    /**
     * @Then /^I change my password from "(?P<old>[^"]*)" to "(?P<new>[^"]*)"$/
     * @Then /^I change my password from "(?P<old>[^"]*)" to "(?P<new>[^"]*)" with confirmation "(?P<confirmation>[^"]*)"$/
     */
    public function iChangeMyPasswordFromToWithConfirmation($currentPassword, $password, $passwordConfirmation = null)
    {
        $this->getPage('Account')->changePassword($currentPassword, $password, $passwordConfirmation);
    }

    /**
     * @Given /^I change my billing address:$/
     */
    public function iChangeMyBillingAddress(TableNode $table)
    {
        $pageInfo = Helper::getPageInfo($this->getSession(), ['controller']);
        if (!\is_array($pageInfo)) {
            Helper::throwException('Could not get page info');
        }
        $pageName = ucfirst($pageInfo['controller']);

        if ($pageName === 'Checkout') {
            $pageName = 'CheckoutConfirm';
        }

        /** @var \Shopware\Tests\Mink\Page\Frontend\Account\Account|CheckoutConfirm $page */
        $page = $this->getPage($pageName);
        $data = $table->getHash();

        $page->changeBillingAddress($data);
    }

    /**
     * @Given /^I change my shipping address:$/
     */
    public function iChangeMyShippingAddress(TableNode $table)
    {
        $pageInfo = Helper::getPageInfo($this->getSession(), ['controller']);
        if (!\is_array($pageInfo)) {
            Helper::throwException('Could not get page info');
        }
        $pageName = ucfirst($pageInfo['controller']);

        if ($pageName === 'Checkout') {
            $pageName = 'CheckoutConfirm';
        }

        /** @var \Shopware\Tests\Mink\Page\Frontend\Account\Account|\Shopware\Tests\Mink\Page\Frontend\Checkout\CheckoutConfirm $page */
        $page = $this->getPage($pageName);
        $data = $table->getHash();

        $page->changeShippingAddress($data);
    }

    /**
     * @Given /^the "([^"]*)" address should be "([^"]*)"$/
     */
    public function theAddressShouldBe($type, $address)
    {
        $this->getPage('Account')->checkAddress($type, $address);
    }

    /**
     * @Given /^I register me:$/
     */
    public function iRegisterMe(TableNode $table)
    {
        $this->getPage('Account')->register($table->getHash());
    }

    /**
     * @When /^I change the payment method to (?P<paymentId>\d+)$/
     * @When /^I change the payment method to (?P<paymentId>\d+):$/
     */
    public function iChangeThePaymentMethodTo($payment, TableNode $table = null)
    {
        $pageInfo = Helper::getPageInfo($this->getSession(), ['controller', 'action']);
        if (!\is_array($pageInfo)) {
            Helper::throwException('Could not get page info');
        }
        $pageName = ucfirst($pageInfo['controller']);

        if ($pageName === 'Checkout') {
            $pageName = ($pageInfo['action'] === 'shippingpayment') ? 'CheckoutCart' : 'CheckoutConfirm';
        }

        /** @var \Shopware\Tests\Mink\Page\Frontend\Account\Account|CheckoutConfirm $page */
        $page = $this->getPage($pageName);
        $data = [
            [
                'field' => 'register[payment]',
                'value' => $payment,
            ],
        ];

        if ($table) {
            $data = array_merge($data, $table->getHash());
        }

        $page->changePaymentMethod($data);
    }

    /**
     * @Then /^the current payment method should be "([^"]*)"$/
     */
    public function theCurrentPaymentMethodShouldBe($paymentMethod)
    {
        $pageInfo = Helper::getPageInfo($this->getSession(), ['controller']);
        if (!\is_array($pageInfo)) {
            Helper::throwException('Could not get page info');
        }
        $pageName = (ucfirst($pageInfo['controller']) === 'Checkout') ? 'CheckoutConfirm' : 'Account';

        $this->getPage($pageName)->checkPaymentMethod($paymentMethod);
    }

    /**
     * @When /^I choose the address "([^"]*)"$/
     */
    public function iChooseTheAddress($name)
    {
        /** @var Account $page */
        $page = $this->getPage('Account');

        $addresses = $this->getMultipleElement($page, 'AddressBox');

        $page->chooseAddress($addresses, $name);
    }

    /**
     * @Given /^I change my profile with "([^"]*)" "([^"]*)" "([^"]*)"$/
     */
    public function iChangeMyProfileWith($salutation, $firstname, $lastname)
    {
        $this->getPage('Account')->changeProfile($salutation, $firstname, $lastname);
    }

    /**
     * @Given /^I should be welcomed with "([^"]*)"$/
     */
    public function iShouldBeWelcomedWith($welcome)
    {
        $welcome = preg_replace("/\s\s+/", ' ', $welcome);

        $assert = new WebAssert($this->getSession());
        $assert->pageTextContains($welcome);
    }

    /**
     * @Given /^there should be an address "([^"]*)"$/
     */
    public function thereShouldBeAnAddress($address)
    {
        /** @var Account|Address $page */
        $page = $this->getPage('Address');

        $address = str_replace('<ignore>', '', $address);
        $testAddress = array_values(array_filter(explode(', ', $address)));

        /** @var array $addressManagementAddressBoxes */
        $addressManagementAddressBoxes = $this->getMultipleElement($page, 'AddressManagementAddressBox');

        /** @var \Shopware\Tests\Mink\Page\Frontend\Address\Elements\AddressManagementAddressBox $box */
        foreach ($addressManagementAddressBoxes as $box) {
            if ($box->containsAdress($testAddress)) {
                return;
            }
        }

        $message = sprintf('Newly created address not found! (%s)', $address);
        Helper::throwException($message);
    }

    /**
     * @Given /^I create a new address:$/
     */
    public function iCreateANewAddress(TableNode $table)
    {
        /** @var Address $page */
        $page = $this->getPage('Address');
        $data = $table->getHash();

        $page->createArbitraryAddress($data);
    }

    /**
     * @Given /^I click "([^"]*)" on address "([^"]*)"$/
     */
    public function iClickOnAddress($locator, $address)
    {
        /** @var Account|Address $page */
        $page = $this->getPage('Address');

        $testAddress = array_values(array_filter(explode(', ', $address)));

        /** @var array $addressManagementAddressBoxes */
        $addressManagementAddressBoxes = $this->getMultipleElement($page, 'AddressManagementAddressBox');

        /** @var AddressManagementAddressBox $box */
        foreach ($addressManagementAddressBoxes as $box) {
            if ($box->containsAdress($testAddress) === false) {
                continue;
            }

            if ($this->endsWith($locator, 'Button')) {
                Helper::pressNamedButton($box, $locator);

                return;
            }

            Helper::clickNamedLink($box, $locator);

            return;
        }

        $message = sprintf('Given address not found! (%s)', $address);
        Helper::throwException($message);
    }

    /**
     * @Then /^I should see only "([^"]*)" with title "([^"]*)"$/
     */
    public function iShouldSeeOnlyWithTitle($address, $addressTitle)
    {
        /** @var Account|Address $page */
        $page = $this->getPage('Address');

        $testAddress = array_values(array_filter(explode(', ', $address)));

        /** @var array $addressManagementAddressBoxes */
        $addressManagementAddressBoxes = $this->getMultipleElement($page, 'AddressManagementAddressBox');

        $addressCount = 0;

        /** @var AddressManagementAddressBox $box */
        foreach ($addressManagementAddressBoxes as $box) {
            if ($box->containsAdress($testAddress) && $box->hasTitle($addressTitle)) {
                ++$addressCount;
            }
        }

        if ($addressCount !== 1) {
            $message = sprintf('Wrong number of given addresses titled "%s" found! Expected 1, found %d.', $addressTitle, $addressCount);
            Helper::throwException($message);
        }
    }

    /**
     * @Given /^I change the current address to:$/
     */
    public function iChangeTheCurrentAddressTo(TableNode $table)
    {
        /** @var \Shopware\Tests\Mink\Page\Frontend\Account\Account|AddressEdit $page */
        $page = $this->getPage('AddressEdit');

        $data = $table->getHash();

        Helper::fillForm($page, 'addressForm', $data);
        Helper::pressNamedButton($page, 'saveAddressButton');
    }

    /**
     * @Given /^I delete the address "([^"]*)"$/
     */
    public function iDeleteTheAddress($address)
    {
        /** @var \Shopware\Tests\Mink\Page\Frontend\Account\Account|Address $page */
        $page = $this->getPage('Address');

        $testAddress = array_values(array_filter(explode(', ', $address)));

        /** @var array $addressManagementAddressBoxes */
        $addressManagementAddressBoxes = $this->getMultipleElement($page, 'AddressManagementAddressBox');

        /** @var AddressManagementAddressBox $box */
        foreach ($addressManagementAddressBoxes as $box) {
            if ($box->containsAdress($testAddress) === false) {
                continue;
            }

            Helper::clickNamedLink($box, 'deleteLink');

            /** @var AddressDelete $page */
            $page = $this->getPage('AddressDelete');

            Helper::pressNamedButton($page, 'confirmDeleteButton');
            break;
        }
    }

    /**
     * @Then /^there must not be an address "([^"]*)"$/
     */
    public function thereMustNotBeAnAddress($address)
    {
        /** @var Account|Address $page */
        $page = $this->getPage('Address');

        $testAddress = array_values(array_filter(explode(', ', $address)));

        /** @var array $addressManagementAddressBoxes */
        $addressManagementAddressBoxes = $this->getMultipleElement($page, 'AddressManagementAddressBox');

        /** @var \Shopware\Tests\Mink\Page\Frontend\Address\Elements\AddressManagementAddressBox $box */
        foreach ($addressManagementAddressBoxes as $box) {
            if ($box->containsAdress($testAddress)) {
                $message = 'Wrong number of given addresses found! Expected 0, found at least 1.';
                Helper::throwException($message);
            }
        }
    }

    /**
     * @Then /^I must not see "([^"]*)" in box with "([^"]*)" title$/
     */
    public function iMustNotSeeInBoxWithTitle($elementName, $title)
    {
        /** @var \Shopware\Tests\Mink\Page\Frontend\Account\Account|\Shopware\Tests\Mink\Page\Frontend\Address\Address $page */
        $page = $this->getPage('Address');

        /** @var array $addressManagementAddressBoxes */
        $addressManagementAddressBoxes = $this->getMultipleElement($page, 'AddressManagementAddressBox');

        /** @var AddressManagementAddressBox $box */
        foreach ($addressManagementAddressBoxes as $box) {
            if ($box->hasTitle($title) && ($box->hasLink($elementName) || $box->hasButton($elementName))) {
                $message = sprintf('Wrong number of boxes with title "%s" and delete button found! Expected 0, found at least 1.', $title);
                Helper::throwException($message);
            }
        }
    }

    private function endsWith($haystack, $needle)
    {
        return preg_match('/' . $needle . '$/', $haystack);
    }
}
